class Timekeeper {
  startTime;
  currentTime;
  endTime;
  running;

  // speed ratio (1 second in real time => x seconds in simulation time)
  speed;

  timerFunc;
  updateInterval = 50; // update canvas clock every 50 ms

  clockTickObservable;
  newTimeObservable;

  clock;
  revealDates;

  constructor(startTime, endTime, speed, revealDates) {
    const { Subject } = rxjs;

    this.startTime = luxon.DateTime.fromISO(startTime);
    this.currentTime = this.startTime;
    this.endTime = luxon.DateTime.fromISO(endTime);

    this.speed = speed;
    this.running = false;

    this.clockTickObservable = new Subject();
    this.newTimeObservable = new Subject();

    this.initClock(this.startTime.toUTC());
    this.revealDates = revealDates;

    this.updateDateTimeAdditionalInfo();
  }

  initClock(startTime) {
    this.clock = new StationClock("clock");
    this.clock.body = StationClock.RoundBody;
    this.clock.dial = StationClock.AustriaStrokeDial;
    this.clock.hourHand = StationClock.BarHourHand;
    this.clock.minuteHand = StationClock.BarSecondHand;
    this.clock.secondHand = StationClock.BarSecondHand;
    this.clock.boss = StationClock.NoBoss;
    this.clock.minuteHandBehavoir = StationClock.BouncingMinuteHand;
    this.clock.secondHandBehavoir = StationClock.CreepingSecondHand;
    this.clock.draw(startTime);
  }

  start() {
    if (!this.timerFunc) {
      this.timerFunc = setInterval(() => {
        this.onTick();
        this.clock.draw(this.currentTime.toUTC());
      }, this.updateInterval);

      this.running = true;
    }
  }

  onTick() {
    this.currentTime = this.currentTime.plus({
      milliseconds: this.speed * this.updateInterval,
    });
    this.clockTickObservable.next(this.currentTime);

    if (this.currentTime >= this.endTime) {
      this.pause();
    }

    this.updateDateTimeAdditionalInfo();
  }

  pause() {
    clearInterval(this.timerFunc);
    this.timerFunc = null;
  }

  reset() {
    this.pause();
    this.currentTime = startTime;
  }

  setSimulationTime(date) {
    this.currentTime = luxon.DateTime.fromISO(date);
    this.newTimeObservable.next(this.currentTime);
  }

  updateDateTimeAdditionalInfo() {
    if (this.revealDates) {
      $("#dateUnderClock").html(
        this.currentTime.toUTC().toFormat("dd/MM/yyyy a")
      );
    }
  }
}
